package com.ruoyi.sim.service.impl;

import java.util.Collections;
import java.util.List;

import cn.ele6.catalyzer.ruoyi.vue.custom.Ele6RYBaseService;
import cn.ele6.catalyzer.ruoyi.vue.enhance.TableDataInfo;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.sim.domain.RealExam;
import com.ruoyi.sim.domain.vo.RealExamCollectionVo;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;
import com.ruoyi.sim.mapper.RealExamCollectionMapper;
import com.ruoyi.sim.domain.RealExamCollection;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.RequestBody;

/**
 * 考试集合Service业务层处理
 *
 * @author tom
 * @date 2024-12-17
 */
@Service
public class RealExamCollectionService extends Ele6RYBaseService {
    @Autowired
    private RealExamCollectionMapper realExamCollectionMapper;

    /**
     * todo:依赖循环的问题。
     */
    @Autowired
    @Lazy
    private CommSendService commSendService;
    @Autowired
    private SocketService socketService;
    @Autowired
    private RealExamService realExamService;

    /**
     * 查询考试集合
     *
     * @param examCollectionId 考试集合主键
     * @return 考试集合
     */
    public RealExamCollection selectRealExamCollectionByExamCollectionId(Long examCollectionId) {
        return realExamCollectionMapper.selectRealExamCollectionByExamCollectionId(examCollectionId);
    }

    /**
     * 查询考试集合列表
     *
     * @param realExamCollection 考试集合
     * @return 考试集合
     */
    public List<RealExamCollection> selectRealExamCollectionList(RealExamCollection realExamCollection) {
        return realExamCollectionMapper.selectRealExamCollectionList(realExamCollection);
    }

    /**
     * 新增考试集合
     *
     * @param realExamCollection 考试集合
     * @return 结果
     */
    public int insertRealExamCollection(RealExamCollection realExamCollection) {
        realExamCollection.setCreateTime(DateUtils.getNowDate());
        return realExamCollectionMapper.insertRealExamCollection(realExamCollection);
    }

    /**
     * 修改考试集合
     *
     * @param realExamCollection 考试集合
     * @return 结果
     */
    public int updateRealExamCollection(RealExamCollection realExamCollection) {
        realExamCollection.setUpdateTime(DateUtils.getNowDate());
        return realExamCollectionMapper.updateRealExamCollection(realExamCollection);
    }

    /**
     * 批量删除考试集合
     *
     * @param examCollectionIds 需要删除的考试集合主键
     * @return 结果
     */
    public int deleteRealExamCollectionByExamCollectionIds(Long[] examCollectionIds) {
        return realExamCollectionMapper.deleteRealExamCollectionByExamCollectionIds(examCollectionIds);
    }

    /**
     * 删除考试集合信息
     *
     * @param examCollectionId 考试集合主键
     * @return 结果
     */
    public int deleteRealExamCollectionByExamCollectionId(Long examCollectionId) {
        return realExamCollectionMapper.deleteRealExamCollectionByExamCollectionId(examCollectionId);
    }

    // -------------------------------- tom add  --------------------------------

    private static final Logger l = LoggerFactory.getLogger(RealExamCollectionService.class);

    /**
     * [教师][分页]查询考试集合列表
     *
     * @param q
     * @return
     */
    public TableDataInfo listExamByTeacher(RealExamCollection q) {
        startPage();
        q.setExamCollectionType(RealExamCollection.Type.EXAM);
        List<RealExamCollectionVo> list = Collections.emptyList();
        realExamCollectionMapper.selectRealExamCollectionList(q)
                .forEach((RealExamCollection s) -> {
                    RealExamCollectionVo t = new RealExamCollectionVo();
                    BeanUtils.copyProperties(s, t);
                    list.add(t);
                });
        return getDataTable(list);
    }

    /**
     * [教师][分页]查询练习集合列表
     *
     * @param p
     * @return
     */
    public TableDataInfo listExerciseByTeacher(RealExamCollection p) {
        startPage();
        p.setExamCollectionType(RealExamCollection.Type.EXERCISE);
        List<RealExamCollectionVo> list = Collections.emptyList();
        realExamCollectionMapper.selectRealExamCollectionList(p).forEach((RealExamCollection s) -> {
            RealExamCollectionVo t = new RealExamCollectionVo();
            BeanUtils.copyProperties(s, t);
            list.add(t);
        });
        return getDataTable(list);
    }

    /**
     * [教师]获取考试集合详细信息
     */
    public AjaxResult getExamInfoByTeacher(Long id) {
        RealExamCollection f = realExamCollectionMapper.
                selectRealExamCollectionByExamCollectionId(id);
        return AjaxResult.success(f);
    }

    /**
     * [教师]获取练习集合详细信息
     */
    public AjaxResult getExerciseInfoByTeacher(Long id) {
        RealExamCollection f = realExamCollectionMapper.
                selectRealExamCollectionByExamCollectionId(id);
        return AjaxResult.success(f);
    }

    /**
     * [教师]新增考试集合
     *
     * @param vo
     * @return
     */
    public AjaxResult addExam(@RequestBody RealExamCollectionVo vo) {
        // check
        // 日期检查
        if (StringUtils.isBlank(vo.getExamCollectionName())) {

        }
        //
        RealExamCollection p = new RealExamCollection();
        BeanUtils.copyProperties(vo, p);
        p.setExamCollectionType(RealExamCollection.Type.EXAM);
        insertRealExamCollection(p);
        return AjaxResult.success();
    }

    /**
     * [教师]新增练习集合
     *
     * @param vo
     * @return
     */
    public AjaxResult addExercise(@RequestBody RealExamCollectionVo vo) {
        // check

        //
        RealExamCollection p = new RealExamCollection();
        BeanUtils.copyProperties(vo, p);
        p.setExamCollectionType(RealExamCollection.Type.EXERCISE);
        insertRealExamCollection(p);
        return AjaxResult.success();
    }

    /**
     * [学院]新增自主练习集合
     * todo:界面入口
     *
     * @param vo
     * @return
     */
    public AjaxResult addSelfExercise(@RequestBody RealExamCollectionVo vo) {
        // check

        // do.
        RealExamCollection p = new RealExamCollection();
        BeanUtils.copyProperties(vo, p);
        p.setExamCollectionType(RealExamCollection.Type.SELF_EXERCISE);
        insertRealExamCollection(p);
        return AjaxResult.success();
    }

    /**
     * [教师]修改考试集合
     *
     * @param vo
     * @return
     */
    public AjaxResult editExam(@RequestBody RealExamCollectionVo vo) {
        // check

        //
        RealExamCollection p = new RealExamCollection();
        BeanUtils.copyProperties(vo, p);
        p.setExamCollectionType(RealExamCollection.Type.EXAM);
        updateRealExamCollection(p);
        // todo:更新关联

        return AjaxResult.success();
    }

    /**
     * [教师]修改练习集合
     *
     * @param vo
     * @return
     */
    public AjaxResult editExercise(@RequestBody RealExamCollectionVo vo) {
        // check

        // do.
        RealExamCollection p = new RealExamCollection();
        BeanUtils.copyProperties(vo, p);
        p.setExamCollectionType(RealExamCollection.Type.EXERCISE);
        updateRealExamCollection(p);
        // todo:更新关联

        return AjaxResult.success();
    }

    /**
     * [教师]删除多个考试集合
     *
     * @param ids
     * @return
     */
    public AjaxResult deleteExams(Long[] ids) {
        // check

        // do.
        deleteRealExamCollectionByExamCollectionIds(ids);
        // todo:更新关联
        return AjaxResult.success();
    }

    /**
     * [教师]删除多个练习集合
     *
     * @param ids
     * @return
     */
    public AjaxResult deleteExercises(Long[] ids) {
        // check

        // do.
        deleteRealExamCollectionByExamCollectionIds(ids);
        // todo:更新关联
        return AjaxResult.success();
    }

    /**
     * [教师]打开考试、训练、练习 集合
     *
     * @param examCollectionId   考试集合Id
     * @param examCollectionType 考试集合类型
     * @return
     */
    @Transactional
    public AjaxResult open(final Long examCollectionId, final String examCollectionType) {
        RealExamCollection rec = selectRealExamCollectionByExamCollectionId(examCollectionId);
        // Check:检查Id有效。
        {
            if (rec == null || rec.getExamCollectionId() == 0L) {
                return AjaxResult.error("考试集合Id不存在!");
            }
            l.debug("rec = {}", rec);
        }
        // Check:检查考试集合类型对应。
        {
            final String dbType = rec.getExamCollectionType();
            if (!StringUtils.equals(dbType, examCollectionType)) {
                return AjaxResult.error("考试集合类型不对应！");
            }
        }
        // Check:检查是否已经打开。
        {
            if (StringUtils.equals(rec.getExamCollectionState(), RealExamCollection.State.OPENED)) {
                return AjaxResult.error("已经打开！");
            }
        }
        //
        switch (examCollectionType) {
            case RealExamCollection.Type.EXERCISE: {
                if (existOpenedByType(RealExamCollection.Type.EXAM)) {
                    return AjaxResult.error("存在已经打开的考试，<br/>请关闭后再操作！");
                }
                closeAllExcludeIdByType(RealExamCollection.Type.EXERCISE, rec.getExamCollectionId());
                closeAllByType(RealExamCollection.Type.SELF_EXERCISE);
                break;
            }
            case RealExamCollection.Type.SELF_EXERCISE: {
                if (existOpenedByType(RealExamCollection.Type.EXAM)) {
                    return AjaxResult.error("存在已经打开的考试，<br/>请关闭后再操作！");
                }
                if (existOpenedByType(RealExamCollection.Type.EXERCISE)) {
                    return AjaxResult.error("存在已经打开的训练，<br/>请关闭后再操作！");
                }
                // 练习可以同时打开多个。
                break;
            }
            case RealExamCollection.Type.EXAM: {
                if (existOpenedByType(RealExamCollection.Type.EXERCISE)) {
                    closeAllByType(RealExamCollection.Type.EXERCISE);
                }
                if (existOpenedByType(RealExamCollection.Type.SELF_EXERCISE)) {
                    closeAllByType(RealExamCollection.Type.SELF_EXERCISE);
                }
                closeAllExcludeIdByType(RealExamCollection.Type.EXAM, rec.getExamCollectionId());
                break;
            }
            default:
                return AjaxResult.error("考试集合类型错误!");
        }
        // Step:尝试打开所有Socket，提前准备，允许有打开失败的
        AjaxResult ar1 = socketService.tryOpenAll();
        if (ar1.isError()) {
            l.info("AjaxResult = {}", ar1);
        }
        // Step:修改考试集合状态。
        {
            rec.setExamCollectionState(RealExamCollection.State.OPENED);
            updateRealExamCollection(rec);
        }
        return AjaxResult.success("开启成功！");
    }

    /**
     * [教师]关闭考试、训练、练习 集合
     *
     * @param examCollectionId   考试集合Id
     * @param examCollectionType 考试集合类型
     * @return
     */
    public AjaxResult close(final Long examCollectionId, final String examCollectionType) {
        RealExamCollection rec = selectRealExamCollectionByExamCollectionId(examCollectionId);
        // Check:检查Id有效。
        {
            if (rec == null || rec.getExamCollectionId() == 0L) {
                return AjaxResult.error("考试集合Id不存在!");
            }
            l.debug("rec = {}", rec);
        }
        // Check:检查考试集合类型对应。
        {
            final String dbType = rec.getExamCollectionType();
            if (!StringUtils.equals(dbType, examCollectionType)) {
                return AjaxResult.error("考试集合类型不对应！");
            }
        }
        // Check:检查是否已经关闭。
        {
            if (StringUtils.equals(rec.getExamCollectionState(), RealExamCollection.State.CLOSED)) {
                return AjaxResult.error("已经关闭！");
            }
        }
        // 处理没有点击交卷，后续没有任何处理的情况。
        if (false) {
            RealExam reQ = new RealExam();
            reQ.setExamCollectionId(examCollectionId);
            realExamService.selectRealExamList(reQ).forEach(re -> {
                realExamService.systemSubmitTimeoutRealExam(re.getExamId());
            });
        }
        // todo:是否还有正在进行的考试
        // 修改为socket常开，直接返回成功结果。
        // Step:修改考试集合状态。
        {
            rec.setExamCollectionState(RealExamCollection.State.CLOSED);
            updateRealExamCollection(rec);
        }
        return AjaxResult.success("关闭成功！");
    }

    public AjaxResult close() {
        closeAllByType(RealExamCollection.Type.EXERCISE);
        closeAllByType(RealExamCollection.Type.SELF_EXERCISE);
        closeAllByType(RealExamCollection.Type.EXAM);
        return AjaxResult.success("所有的考试、训练、练习都关闭成功！");
    }

    public List<RealExamCollection> listAllByType(String examCollectionType) {
        RealExamCollection q = new RealExamCollection();
        q.setExamCollectionType(examCollectionType);
        return selectRealExamCollectionList(q);
    }

    /**
     * 是否有一个open的任何类型的集合。
     *
     * @return
     */
    public boolean existAtLeastOneOpened() {
        return (selectRealExamCollectionOpened() != null);
    }

    public boolean existOpenedByType(String examCollectionType) {
        RealExamCollection q = new RealExamCollection();
        q.setExamCollectionType(examCollectionType);
        q.setExamCollectionState(RealExamCollection.State.OPENED);
        List<RealExamCollection> list = selectRealExamCollectionList(q);
        return (!list.isEmpty());
    }

    public RealExamCollection selectRealExamCollectionOpened() {
        RealExamCollection q = new RealExamCollection();
        q.setExamCollectionState(RealExamCollection.State.OPENED);
        List<RealExamCollection> list = selectRealExamCollectionList(q);
        if (list != null && !list.isEmpty()) {
            return list.get(0);
        }
        return null;
    }

    /**
     * 关闭所有 考试、训练、练习
     *
     * @return
     */
    public AjaxResult closeAll() {
        closeAllByType(RealExamCollection.Type.EXERCISE);
        closeAllByType(RealExamCollection.Type.SELF_EXERCISE);
        closeAllByType(RealExamCollection.Type.EXAM);
        return AjaxResult.success("所有考试、训练、练习的集合被关闭");
    }

    /**
     * 强制停掉所有的
     *
     * @param type
     */
    private void closeAllByType(final String type) {
        RealExamCollection q = new RealExamCollection();
        q.setExamCollectionType(type);
        selectRealExamCollectionList(q)
                .forEach((RealExamCollection ec) -> {
                    ec.setExamCollectionState(RealExamCollection.State.CLOSED);
                    updateRealExamCollection(ec);
                });
    }

    /**
     * 强制停掉 除本type外的 所有的type
     *
     * @param type
     * @param excludeRecId
     */
    private void closeAllExcludeIdByType(final String type, final Long excludeRecId) {
        List<RealExamCollection> list = listAllByType(type);
        for (RealExamCollection ec : list) {
            if (ec.getExamCollectionId().equals(excludeRecId)) {
                // 跳过本考试。
                continue;
            }
            ec.setExamCollectionState(RealExamCollection.State.CLOSED);
            updateRealExamCollection(ec);
        }
    }

    /**
     * @return 可以为null。
     */
    public RealExamCollection getCurrentExamCollection() {
        RealExamCollection q = new RealExamCollection();
        q.setExamCollectionState(RealExamCollection.State.OPENED);
        List<RealExamCollection> list = selectRealExamCollectionList(q);
        if (list == null || list.isEmpty()) {
            return null;
        }
        return list.get(0);
    }
}
